%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function calculates the contribution [mill. USD] of taking a 
% particular action. The contribution is based on the mission selected 
% (depending of the vessels technical capabilities), the lengt of the 
% contract (i.e. strategy), and the state of the market
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [contribution,con_reward,mission_current] = Contri(probwin,SellingCost, market_current,OpDays, SwitchDays,ii, Comp,  NN_Space_Mission, Sample_DayRates, strategy_current, tt_current, switch_cost, design_previous, design_current,disc)

if strategy_current == 3 % Sell
    con_reward = SellingCost(market_current); % Reward
    con_switch = 0; % Switching costs 
    mission_current = 0; % Mission taken
    % Estimate contribution 
    contribution = (con_reward - con_switch)*(1/(1+disc)^(tt_current-0)); 

else
    
    % Find out if the contracts are won or lost
    win = zeros(1,NN_Space_Mission); 
    for mm = 1:(NN_Space_Mission - 1)
        r = rand; % Pick a random number in [0,1]
        p = sum(r >= cumsum(probwin(market_current,:)));
        if p == 1
            win(1,mm) = 1; % Contract is won
        else
            win(1,mm) = 0; % Contract is lost 
        end
    end
    win(1,9) = 1; % Lay-up is always an option 
    
    % Calculate the reward for each mission 
    Days = OpDays - SwitchDays;
    Reward = []; 
    for mm = 1:NN_Space_Mission
        if strategy_current == 1 % Short-term contract
            % If design is not compitable with mission (mm) and the Dayrate is negatice
            if Comp(mm) == -Inf && Sample_DayRates{1,strategy_current}(mm,tt_current) < 0 
                Reward(mm) = -(win(1,mm)*Comp(mm)*Days*Sample_DayRates{1,strategy_current}(mm,tt_current));
            else % dayrate is positive 
                Reward(mm) = win(1,mm)*Comp(mm)*Days*Sample_DayRates{1,strategy_current}(mm,tt_current);
            end
        else % Long-term contract 
            % If design is not compitable with mission (mm) and the Dayrate is negatice
            if Comp(mm) == -Inf && Sample_DayRates{1,strategy_current}(mm,tt_current) < 0
                Reward(mm) = -(win(1,mm)*Comp(mm)*Days*Sample_DayRates{1,strategy_current}(mm,tt_current)+Comp(mm)*OpDays*Sample_DayRates{1,strategy_current}(mm,tt_current)*(1/(1+disc)^1)+Comp(mm)*OpDays*Sample_DayRates{1,strategy_current}(mm,tt_current)*(1/(1+disc)^2));
            else
                Reward(mm) = win(1,mm)*Comp(mm)*Days*Sample_DayRates{1,strategy_current}(mm,tt_current)+Comp(mm)*OpDays*Sample_DayRates{1,strategy_current}(mm,tt_current)*(1/(1+disc)^1)+Comp(mm)*OpDays*Sample_DayRates{1,strategy_current}(mm,tt_current)*(1/(1+disc)^2);
            end
        end
    end

    % Find and take the avaiable mission with highest reward
    [con_reward, mission_current] = max(Reward);

    % Fint the contribution of being in that particular statee (Step 2. c) 
    con_switch = switch_cost{ii}(design_previous, design_current); % Switching cost 

    % If the last time-step is reached, the sunset value is calculated
    if  (tt_current == 11 && strategy_current == 1) || (tt_current == 8 && strategy_current == 2)
        SellC = SellingCost(market_current);
    else
        SellC = 0;
    end

    contribution = (con_reward - con_switch + SellC)*(1/(1+disc)^(tt_current-0));
end
end 