%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function calculates avaiable deck area, freeboard, deadweight and GM
% for a design. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Space_Design_Fix_full, Lightweight] = Design_Information(Space_Design_Var_full,N_Space_Design_Var_full, N_Var_Design, Length, Beam,Depth, F_min)

Space_Design_Fix_full = zeros(N_Space_Design_Var_full,1); %[Deck Area, Freeboard, Deadweigth, GM]

%%% Initialize data
% Deck area [m2] requirement for each topside equipment
DeckArea_Eq(1,:) = xlsread('Case.xlsx',6,'B4:D4'); % ACC
DeckArea_Eq(2,:) = xlsread('Case.xlsx',6,'B5:D5'); % MC
DeckArea_Eq(3,:) = xlsread('Case.xlsx',6,'B6:D6'); % LWI
DeckArea_Eq(4,:) = xlsread('Case.xlsx',6,'B7:D7'); % ROV
DeckArea_Eq(5,:) = xlsread('Case.xlsx',6,'B8:D8'); % PC
DeckArea_Eq(6,:) = xlsread('Case.xlsx',6,'B9:D9'); % MP

% Weight [tonn] for each design parameter
Weigth_Eq(1,:) = xlsread('Case.xlsx',6,'B13:D13'); % ACC
Weigth_Eq(2,:) = xlsread('Case.xlsx',6,'B14:D14'); % MC
Weigth_Eq(3,:) = xlsread('Case.xlsx',6,'B15:D15'); % LWI
Weigth_Eq(4,:) = xlsread('Case.xlsx',6,'B16:D16'); % ROV
Weigth_Eq(5,:) = xlsread('Case.xlsx',6,'B17:D17'); % PC
Weigth_Eq(6,:) = xlsread('Case.xlsx',6,'B18:D18'); % MP

% Center of gravity [m] for each design parameter
CoG_Eq(1,:) = xlsread('Case.xlsx',6,'B22:D22'); % ACC
CoG_Eq(2,:) = xlsread('Case.xlsx',6,'B23:D23'); % MC
CoG_Eq(3,:) = xlsread('Case.xlsx',6,'B24:D24'); % LWI
CoG_Eq(4,:) = xlsread('Case.xlsx',6,'B25:D25'); % ROV
CoG_Eq(5,:) = xlsread('Case.xlsx',6,'B26:D26'); % PC
CoG_Eq(6,:) = xlsread('Case.xlsx',6,'B27:D27'); % MP

% Scaling factors [-]
Deck_Area_koefficient = xlsread('Case.xlsx',6,'C30');      
Lightweighth_Koefficient = xlsread('Case.xlsx',6,'C31');     

% Various constands
rho_water = xlsread('Case.xlsx',6,'C32');   % Sea water density [tonn/m3]
Cb = xlsread('Case.xlsx',6,'C33');   % Block Coefficient [-]

%%% Estimating Avaiable Deck [m2] for each design configuration 
% Estimate the avaiable Deck area for a vessel without any topside ...
% equipment installed: 
Space_Design_Fix_full(:,1) = Deck_Area_koefficient*Length*Beam; 

% Subtract the required deck area for each equipment installed 
for ss = 1: N_Space_Design_Var_full
    for vv = 1: N_Var_Design
        Space_Design_Fix_full(ss,1) = Space_Design_Fix_full(ss,1) - ...
            DeckArea_Eq(vv,Space_Design_Var_full(ss,vv));
    end
end

%%% Estimating lightship weight [tonnes] for each design configuration 
% Estimate the  ligtship  weigth of the vessel without any topside ...
% equipment installed  
Weighth_Design(1:N_Space_Design_Var_full,1) = ...
    Lightweighth_Koefficient*Length*Beam*Depth; 

Lightweight = Weighth_Design;

% Add the weight for each equipment installed
for ss = 1: N_Space_Design_Var_full
    for vv = 1: N_Var_Design
        Weighth_Design(ss,1) = Weighth_Design(ss,1) + ...
            Weigth_Eq(vv,Space_Design_Var_full(ss,vv));
    end
end

%%% Estimating the Volume displacement for each vessel configuration
Volume_Displacement(:,1) = Weighth_Design(:,1)./rho_water;  

%%% Estimating the draught for each vessel configuration 
T(:,1) = Volume_Displacement(:,1)./(Cb.*Length.*Beam); 

%%% Estimating the freeboard for each vessel configuration 
Space_Design_Fix_full(:,2) = Depth - T(:,1); 

%%% Estimating Deadweight for each design configuration 
Space_Design_Fix_full(:,3) = rho_water.*Cb.*(Length.*Beam).*...
    (Depth-F_min) - Weighth_Design(:,1);

%%% Estimating GM for each design configuration 
KB(:,1) =  T(:,1)/2; 
BM(:,1) = (Beam^2)./(12.*T(:,1));
sum_Weight_CoG(1:N_Space_Design_Var_full,1) =  (Lightweighth_Koefficient*Length*Beam*Depth)*(Depth/2);
for ss = 1: N_Space_Design_Var_full
    for vv = 1: N_Var_Design
        sum_Weight_CoG(ss,1) = sum_Weight_CoG(ss,1) + CoG_Eq(vv,Space_Design_Var_full(ss,vv))*Weigth_Eq(vv,Space_Design_Var_full(ss,vv)); 
    end
    KG(ss,:) = sum_Weight_CoG(ss,1)/Weighth_Design(ss,1);
end

Space_Design_Fix_full(:,4) = KB(:,1) + BM(:,1) - KG(:,1);
end
