%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Q-Learning Algorithm. The script follows the generic Q-learning algorith
% presented in the thesis. The output of this algorithm is the Policy 
% indicating which action to take for each state-time combination. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Max number of interations (Step. 0b in generic algorithm)
N_Interations_Q = 100; 

% Initialize Matrixs for Further analysis 
Q_SumSum = cell(N_Design,1); % Sum of Q-Matrix  
        
 con_total_Q = cell(N_Design,1); % Life-cycle contribution
 for nn = 1:N_Design
   for ii = 1:N_Interations_Q
    con_total_Q{nn}(ii,1) = 0;
   end
end

for ii = 1:N_Design % Spesify which design(s) to analyse 
    
    % Initialize Q-Matrix (Step. 0a in generic algorithm)
    Q = cell(1,Lifetime + 2);
    for tt = 1:Lifetime + 2
        Q{1,tt} = zeros(N_Space_State,N_Space_Action);
    end
    
     % Calculate Switching time 
     SwitchDays = SDays(ii);
        
    iter = 1; % Set interation counter (Step 0c. in generic algorithm)
    
while iter  <= N_Interations_Q 
    
    % Output information 
    fprintf('Design nr. %d, Interation nr. %d\n',ii, iter); 
    
    % Initialie Starting State (Step 0d. in generic algorithm)
    state_idx_action = state_idx_start;
    
    % Sample market state and requirement state for the given interation (Step. 1 in generic algorithm) 
    [Sample_Market, Sample_Requirement] = Sample_Path(Lifetime, TM_Requirement); 
 
    % Sample dayrates for the given interation (Step. 1 cont. in generic algorithm)
    [Sample_DayRates] = Sample_Dayrates(Sample_Market, Lifetime,N_Strategy,N_Space_Mission,ScalingFactor,Dayrates_Data);
    
    % Select Learnign Rate
    if iter == 1
        alpha = 1;
    else
        alpha = 0.7;
    end
        
    % Simulate the vessel's lifetime (Step 2 in generic algorithm)
    tt_current = 2; % 2 represents year 5, 3 represents year 6, etc.
    while tt_current < Lifetime + 2 
        
        %%% Select action (Step 2a. in generic algorithm)
        tt_action = tt_current - 1; % Time when action is taken
        [action_idx, tt_next] = ActionSelection(tt_action,Q, iter, N_Interations_Q, tt_current, Lifetime, time_long, time_short, state_idx_action, Space_Action, N_Space_Action);

        %%% Simulate outcome of action (Step 2b. in generic algorithm)   
        % I.e. find current state (stochastic parameter) 
        state_idx_current = find(Space_State(:,1) == Space_Action(action_idx,1) & Space_State(:,2) == Space_Action(action_idx,2) & Space_State(:,3) == Sample_Market(tt_current) & Space_State(:,4) == Sample_Requirement(tt_current));
          
        % Obtain information of previous and current states 
        design_previous = Space_State(state_idx_action,1);
        design_current = Space_State(state_idx_current,1);
        strategy_current = Space_State(state_idx_current,2);
        market_current = Space_State(state_idx_current,3);
        requirement_current = Space_State(state_idx_current,4);
        
        % Estimate immediate Contribution (Step. 2c). 
        [Comp] = Mission_Compatability(N_Space_Mission,N_Var_Design,Space_Design,Requirement_Matrix,requirement_current,design_current);
        [contribution,rew,miss] = Contri(probwin, SellingCost, market_current,OpDays, SwitchDays,ii, Comp, N_Space_Mission, Sample_DayRates, strategy_current, tt_current, switch_cost, design_previous, design_current, disc);
        con_total_Q{ii}(iter,1) = con_total_Q{ii}(iter,1) + contribution;
        
        % Q learning 
        Q{1,tt_action}(state_idx_action,action_idx) = (1-alpha)*Q{1,tt_action}(state_idx_action,action_idx) + alpha*(contribution + gamma*max(Q{1,tt_current}(state_idx_current,:)));
        
        % Prepeare next step 
        state_idx_action = state_idx_current; 

        % Change time step
        tt_current = tt_current + tt_next;
        
    end % while tt < Lifetime
        
        % Update information 
        for tt = 2:Lifetime + 2
            Q_SumSum{ii}(iter,tt) = sum(sum(Q{1,tt}));
        end

        % Update interation counter (Step 3. in generic algorithm)
        iter = iter + 1; 
end
    % Store information 
    if ii == 1
        Q_Design_1 = Q; % Changeability level 0
    elseif ii == 2
        Q_Design_2 = Q; % Changeability level 1
    else % ii == 3
        Q_Design_3 = Q; % Changeability level 2
    end
end

% Extract Policy and Value Function (Step 4. in generic algrithm), 
% in addition to design and strategy for further analysis 
Policy = cell(N_Design,1);
Value_function = cell(N_Design,1); 
Design = cell(N_Design,1);
Strategy = cell(N_Design,1);

for ii = 1:N_Design
    if ii == 1
        Q_post = Q_Design_1;
    elseif ii == 2
        Q_post = Q_Design_2;
    else % ii == 3
        Q_post = Q_Design_3;
    end
    for tt_act = 1:Lifetime
        for ss = 1:N_Space_State
            [Value_function{ii}(tt_act,ss), Policy{ii}(ss,tt_act)] = max(Q_post{1,tt_act}(ss,:));
            Design{ii}(ss,tt_act+1) = Space_Action(Policy{ii}(ss,tt_act), 1);
            Strategy{ii}(ss,tt_act+1) = Space_Action(Policy{ii}(ss,tt_act), 2);
        end
    end
end
